{% spaceless %}
    {% with prop.classes as classes  %}
        {% if title %}<h5>{% if "collapse" in classes %}<a href="#" data-bs-toggle="collapse" data-bs-target="#form{{ uid }}-s{{ section }}" aria-expanded="{% if "show" in classes %}true{% else %}false{% endif %}" aria-controls="collapseExample">{% endif %}{{ title }}{% if "collapse" in classes %}</a>{% endif %}</h5>{% endif %}
        {% if prop.desription %}<p>{{ prop.description }}</p>{% endif %}
        <div id="form{{ uid }}-s{{ section }}" class="{{ classes|join:" " }}">
        {% for form_field in prop.fields %}
            {% include "djangocms_form_builder/bootstrap5/render/field.html" with form=form form_field=form_field %}
        {% endfor %}
        </div>
    {% endwith %}
{% endspaceless %}
