form .form-row div.frontend-button-group.frontend-button-group-block{display:flex;flex-wrap:wrap}form .form-row div.frontend-button-group.frontend-button-group-block label{overflow:hidden;white-space:nowrap;text-overflow:ellipsis;flex-basis:calc(25% - 6px)}@media(min-width: 820px){form .form-row div.frontend-button-group.frontend-button-group-block label{flex-basis:calc(20% - 6px)}}form .form-row div.frontend-button-group label.btn-grp{margin:3px;padding:4px 7px;text-transform:none;text-align:center !important;outline:2px solid transparent;width:auto !important;font-weight:normal !important}form .form-row div.frontend-button-group input[property=color]+label.btn-grp[style="--fe-value: ;"]{color:var(--dca-black, #333) !important}@media(prefers-color-scheme: dark){form .form-row div.frontend-button-group input[property=color]+label.btn-grp[style="--fe-value: ;"]{color:var(--dca-black, #eee) !important}}form .form-row div.frontend-button-group input[property=color][value=transparent]+label.btn-grp{color:var(--dca-black, #333) !important}@media(prefers-color-scheme: dark){form .form-row div.frontend-button-group input[property=color][value=transparent]+label.btn-grp{color:var(--dca-black, #eee) !important}}form .form-row div.frontend-button-group input[property=text]+label.btn-grp{color:var(--dca-black, #333) !important}@media(prefers-color-scheme: dark){form .form-row div.frontend-button-group input[property=text]+label.btn-grp{color:var(--dca-black, #eee) !important}}form .form-row div.frontend-button-group input[type=radio]:checked+label.btn-grp,form .form-row div.frontend-button-group input[type=checkbox]:checked+label.btn-grp{outline:2px solid #0bf;border-color:#fff;border-radius:0}form .form-row div.frontend-button-group input[property=text]:checked+label.btn-grp,form .form-row div.frontend-button-group input[property=icon]:checked+label.btn-grp{background:#0bf}form .form-row div.frontend-button-group input[property=opacity]+label.btn-grp{width:3.5em !important;padding-left:0 !important;padding-right:0 !important;color:var(--dca-black, #eee) !important;overflow:hidden;white-space:nowrap;background:rgba(var(--bs-secondary-rgb), calc(var(--fe-value) / 100))}@media(prefers-color-scheme: light){form .form-row div.frontend-button-group input[property=opacity][value="50"]+label.btn-grp,form .form-row div.frontend-button-group input[property=opacity][value="25"]+label.btn-grp,form .form-row div.frontend-button-group input[property=opacity][value="10"]+label.btn-grp{color:#000 !important}}form .form-row div.frontend-button-group input[property=link-size][value=btn-lg]+label.btn-grp{padding:.5rem 1rem !important;font-size:1.25rem;border-radius:.3rem}form .form-row div.frontend-button-group input[property=link-size][value=btn-sm]+label.btn-grp{padding:.25rem .5rem !important;font-size:.875rem;border-radius:.2rem}form .form-row div.frontend-button-group input[property=list_state]+label.btn-grp[style="--fe-value: ;"]{color:var(--dca-black, #333) !important}@media(prefers-color-scheme: dark){form .form-row div.frontend-button-group input[property=list_state]+label.btn-grp[style="--fe-value: ;"]{color:var(--dca-black, #eee)}}form .form-row div.frontend-button-group input[property=list_state][value=active]+label.btn-grp{background:var(--bs-primary);color:#fff}form .form-row div.frontend-button-group input[property=list_state][value=disabled]+label.btn-grp{color:var(--bs-gray-600) !important;background:#fff}form .form-row div.frontend-button-group input[property=shadow]+label.btn-grp{margin-right:1em;width:3.5em !important;padding-left:0 !important;padding-right:0 !important;background:var(--bs-secondary);color:#fff !important;text-align:center !important;border:var(--bs-secondary) 1px solid !important;overflow:hidden;white-space:nowrap}form .form-row div.frontend-button-group input[property=shadow][value=lg]+label.btn-grp{box-shadow:0 1rem 3rem rgba(0,0,0,.175) !important}form .form-row div.frontend-button-group input[property=shadow][value=reg]+label.btn-grp{box-shadow:0 .5rem 1rem rgba(0,0,0,.15) !important}form .form-row div.frontend-button-group input[property=shadow][value=sm]+label.btn-grp{box-shadow:0 .125rem .25rem rgba(0,0,0,.075) !important}form .form-row div.frontend-button-group input[property=nav-design][value=light]+label.btn-grp{background:var(--bs-light);color:var(--bs-gray-900) !important}form .form-row div.frontend-button-group input[property=nav-design][value=dark]+label.btn-grp{background:var(--bs-dark);color:var(--bs-white)}form .form-row div.frontend-button-group .btn-white{background:#fff;color:#000 !important}form .form-row div.frontend-button-group .btn-light{color:#000 !important}form .form-row div.frontend-button-group .btn-dark{color:#fff !important}form .form-row div.frontend-button-group .btn-transparent{color:var(--dca-black, var(--body-fg, #000))}body:not(.djangocms-admin-style) .frontend-button-group .optgroup{clear:left}/*# sourceMappingURL=button_group.css.map */
