{% load static cms_tags sekizai_tags i18n %}
{% spaceless %}
   {% if form %}
        <form id="form{{ uid }}"
              class="djangocms-form-builder-ajax-form"
              novalidate action="{% url 'form_builder:ajaxview' instance.id %}"
              method="post">
            {% csrf_token %}
            {% include 'djangocms_form_builder/ajax_form.html' with form=form instance=instance tracking=instance.tracking_code RECAPTCHA_PUBLIC_KEY=RECAPTCHA_PUBLIC_KEY %}
            <input type="submit" value="{{ instance.form_submit_message|default:_("Submit") }}"
                class="btn btn-{{ instance.form_submit_context|default:"primary" }}">
        </form><div class="clearfix"></div>
    {% endif %}
{% endspaceless %}
